
/*===============================<o>=====================================

  This software was written by Ian L. Kaplan, Chief Fat Bear, Bear Products
  International.  Use of this software, for any purpose, is granted on two
  conditions: 

   1. This copyright notice must be included with the software
      or any software derived from it.

   2. The risk of using this software is accepted by the user.  No
      warranty as to its usefulness or functionality is provided
      or implied.  The author and Bear Products International provides
      no support.

   May 1996

 ===============================<o>=====================================*/


//
// For documentation on openGL see 
//
//  "OpenGL Programming Guide: The Official guide to Learning OpenGL
//   Release 1" by Neider Davis and Woo, Addison Wesley.
//

// Microsoft Windows/NT openGL include files
#include "glos.h"
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glaux.h>
#include <stdio.h>
#include "sier.h"

//
// Display list name
//

static const GLuint SIERPINSKI_LIST = 1;

//
// The "display" callback function needs to access the point list
// so this is a global, static, variable.
//
static int *local_list = NULL;
static int local_num_faces;

//
// Ideally all the openGL display functions would be encapsulated inside
// of an openGL display class.  However class functions cannot be used
// as call back functions.  So the functions below are written in C.
//

typedef struct { float min;
                 float max; 
               } AxisMinMax;

static AxisMinMax Axis = {0.0, 0.0 };


static void init_minmax( int max )
{
  const int extra = 5;

  Axis.max = (float)(max + extra);
} // init_minmax



//
// The glRotatef function takes four arguments, the angle of rotation,
// and a 3-D point.  The axis of rotation defined by the line from (0,0,0)
// to the 3-D point.  So to rotate on the X-axis, the point would be
// (1, 0, 0), to rotate on the Y-axis (0, 1, 0) and to rotate on the
// Z-axis (0, 0, 1).
//
const int On  = 1;
const int Off = 0;
// Rotate first on the X-axis
static int AxisPoint[ 3 ] = {On, Off, Off};
static int AxisIx = 0;

static int SpinAngle = 0;       // spin angle, in degrees


static int point_neq(int *a, int *b) 
{ 
  return (a[0] != b[0] || a[1] != b[1] || a[2] != b[2]);
} // point_neq



//
// Display the wire frame described by the face list pointed to by
// CurObj.
//
void CALLBACK display(void)
{
  glClear(GL_COLOR_BUFFER_BIT);
  
  glPushMatrix();
  
  glRotatef((float)SpinAngle, (float)AxisPoint[0],
	    (float)AxisPoint[1],
	    (float)AxisPoint[2]);

  glCallList( SIERPINSKI_LIST );
  
  glPopMatrix();
  glFlush();
  auxSwapBuffers();
}  // callback function display 




void CALLBACK spinDisplay(void)
{
  SpinAngle = SpinAngle + 2;
  if (SpinAngle > 360) {
    AxisPoint[ AxisIx ] = Off;
    AxisIx = (AxisIx + 1) % 3;
    AxisPoint[ AxisIx ] = On;
    SpinAngle = SpinAngle - 360;
  }
  display();
}



void CALLBACK Reshape(GLsizei w, GLsizei h)
{
  glViewport(0, 0, w, h);

  glMatrixMode (GL_PROJECTION);
  glLoadIdentity();

  if (w <= h) {
    float AdjY = Axis.max * (GLfloat)h / (GLfloat)w;
    glOrtho(-Axis.max, Axis.max,  // x-axis
	    -AdjY, AdjY,          // y-axis
	    -Axis.max, Axis.max); // z-axis
  }
  else {
    float AdjX = Axis.max * (GLfloat)h / (GLfloat)w;
    glOrtho(-AdjX, AdjX,          // x-axis
	    -Axis.max, Axis.max,  // y-axis
	    -Axis.max, Axis.max); // z-axis
  }

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
} /* callback function Reshape */




static void build_display_list(void)
{
  typedef int point[ 3 ];
  const int size_of_point = sizeof( point )/sizeof(int);
  const int points_per_face = 4;
  int *tmp;
  int i, j;

  tmp = local_list;
  glNewList( SIERPINSKI_LIST, GL_COMPILE );

  for (j = 0; j < local_num_faces; j++) {
    glBegin( GL_POLYGON );
    for (i = 0; i < points_per_face; i++) {
      if (i < points_per_face-1 || point_neq( tmp, tmp - size_of_point)) {
	glVertex3iv(tmp);
      }
      tmp += size_of_point;
    } // i
    glEnd();
  } // j

  glEndList();
} /* build_display_list */




static void initGL(void) 
{
  const char *WindowName = "Sierpinski Pyramid";

  // GL display stuff
  auxInitDisplayMode (AUX_DOUBLE | AUX_RGBA );
  auxInitPosition (0, 0, 500, 500);
  auxInitWindow(WindowName);

  //  clear background to black 
  glClearColor (0.0, 0.0, 0.0, 0.0);   
  // draw all lines in white, on the black background
  glColor3f(1.0, 1.0, 1.0);
  // Set polygon mode to line
  glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
  // "cull" invisible polygons
  // glCullFace(GL_BACK);
  // turn on culling
  // glEnable(GL_CULL_FACE);


  // Build the openGL display list for the 
  // sierpinsky pyramid.
  build_display_list();

  auxReshapeFunc(Reshape);
  auxIdleFunc(spinDisplay);
  auxMainLoop(display);

}  // initGL



void openGL_display( const int *facelist, const int pyr_size, 
		     int const num_faces )
{
  local_list = (int *)facelist;
  local_num_faces = num_faces;
  init_minmax( pyr_size );

  initGL();
}
